#ifndef EMAP_H
#define EMAP_H
#include "config.h"
#include "layout.h"
#include "obj.h"

// !!! 現状で32ビット使い切っている
// electricなど追加したいので、その場合は
// - en_itemを廃止してがっががen_waterに反応するようにする?
// - en_vegetableはなくせるか？
// - shockをblast_left/rightで兼用?
//      - gaggaのshock/blastの効果範囲が同一になる(実害ある?)
//      - mingoのつつきでふっとばしが発生しないよう、set_emap_mingoで
//        「壁をつつく場合」のみEM_BLASTを設定する必要がある
typedef enum {
	en_meat,          // 動物。おじぎり草が捕食
	en_gondin,        // ごんでぃんの実
    en_sign,          // 立て札
	en_weight,        // スイッチに対する重量
    en_teleport,      // テレポート場

	// ここより上はOwnerを使用するため、最初にまとめておく
    en_ownered_end,

    en_chee = en_ownered_end,
	en_vegetable,     // 植物(おじぎり草、ごんでぃんの実) 
//	en_item,          // 非生物のもの(がっがは生物同様反応。現在はしぐくのみ)
	en_block_left,  // その位置よりその方向への移動を禁止する
	en_block_right, //   壁などの周囲に設定する
    en_block_up,
    en_block_down,
    en_exclude_left,  // 障害物内部からその方向に押し出そうとする
    en_exclude_right, 
    en_exclude_up,
    en_exclude_down,

	en_push_left,   // 左に動かそうとする(左へ移動中の動く床の上部など)
    en_push_right,  // 右に動かそうとする(右へ移動中の動く床の上部など)
    en_push_up,     // 上や下に動かそうとする(上下へ移動中のリフトなど)
    en_push_down,   //   実際にはexclude_upや重力を利用するためpush_left/rightのような
	                //   効果は発揮しない。ジャンプ時の上下慣性判定のため
					//   リフト上部にこの属性を設定する。

    en_push_leftdown, // 左下・右下に動かそうとする。斜面用。
    en_push_rightdown,// ジャンプ不能など特別扱いのため、left+downでは代用できない
	en_blast_left,
	en_blast_right,

	en_goal,          // 開いたドア
    //    en_eat,           // Gondinの実を食べようとするキャラの存在(req_queue使って!!廃止?)
    en_shock,         // 衝撃(ヒビ壁を破壊する、リフトやパンチの進行方向に発生する)
    en_flame,         // 焔
	en_electric,      // 電撃
    en_water,         // 水
    en_ice,           // 氷
    en_pierce,        // 棘
    en_fwall,         // ヒビ壁

    en_end
} effect_no_t;

typedef unsigned int emap_e_t;

#define EM_MEAT (1 << en_meat)
#define EM_GONDIN (1 << en_gondin)
#define EM_SIGN (1 << en_sign)
#define EM_WEIGHT (1 << en_weight)
#define EM_TELEPORT (1 << en_teleport)

#define EM_CHEE (1 << en_chee)
#define EM_VEGETABLE (1 << en_vegetable)
//#define EM_ITEM (1 << en_item)
#define EM_BLOCK_LEFT (1 << en_block_left)
#define EM_BLOCK_RIGHT (1 << en_block_right)
#define EM_BLOCK_UP (1 << en_block_up)
#define EM_BLOCK_DOWN (1 << en_block_down)
#define EM_EXCLUDE_LEFT (1 << en_exclude_left)
#define EM_EXCLUDE_RIGHT (1 << en_exclude_right)
#define EM_EXCLUDE_UP (1 << en_exclude_up)
#define EM_EXCLUDE_DOWN (1 << en_exclude_down)
#define EM_PUSH_LEFT (1 << en_push_left)
#define EM_PUSH_RIGHT (1 << en_push_right)
#define EM_PUSH_UP (1 << en_push_up)
#define EM_PUSH_DOWN (1 << en_push_down)
#define EM_PUSH_LEFTDOWN (1 << en_push_leftdown)
#define EM_PUSH_RIGHTDOWN (1 << en_push_rightdown)
#define EM_BLAST_LEFT (1 << en_blast_left)
#define EM_BLAST_RIGHT (1 << en_blast_right)
#define EM_GOAL (1 << en_goal)
//#define EM_EAT (1 << en_eat)
#define EM_SHOCK (1 << en_shock)
#define EM_FLAME (1 << en_flame)
#define EM_ELECTRIC (1 << en_electric)
#define EM_WATER (1 << en_water)
#define EM_ICE (1 << en_ice)
#define EM_PIERCE (1 << en_pierce)
#define EM_FWALL (1 << en_fwall)
//#define EM_ (1 << en_)



#define NextEMap (-(~(-current_emap)))

// emapチェック時に引っかかった座標をエンコードしてCheckValとする
// その相互変換マクロ
// マップ周囲は壁だから、(0, 0)が引っかかることはないと想定
#define Pos2CheckVal(x, y) (((y)<<8) + (x))
#define CheckVal2PosX(v) (v & 0xff)
#define CheckVal2PosY(v) (((v)>>8) & 0xff)
#define ReturnCheckEMapPos(x, y, eb) \
	if (CheckEMap(x, y, eb)){return Pos2CheckVal(x, y);}
//{assert(GetEMapOwner(x, y, eb) != NULL);      

// emapアクセスマクロ
#define SetEMapStatic(x, y, em) \
    (emap_static[y][x] |= (em))
#define ResetEMapStatic(x, y, em) \
    (emap_static[y][x] &= ~(em))
#define SetEMap(x, y, em) \
    (emap[NextEMap][y][x] |= (em))
#define ResetEMap(x, y, em) \
    (emap[NextEMap][y][x] &= ~(em))
#define CheckEMap(x, y, em) \
    (emap[current_emap][y][x] & (em))
#define SetEMapOwnerStatic(x, y, en, obj) \
    (emap_owner_static[en][y][x] = obj)
#define ResetEMapOwnerStatic(x, y, en) \
    (emap_owner_static[en][y][x] = NULL)
#define SetEMapOwner(x, y, en, obj) \
	(emap_owner[NextEMap][en][y][x] = obj)
#define ResetEMapOwner(x, y, en) \
	(emap_owner[NextEMap][en][y][x] = NULL)
// Ownerのアクセス頻度は低いので
// emap_owner_staticを毎回コピーせずに、emap_ownerで見つからないときだけ見に行く
// ※enとEM混同防止のため、GetEMapOwnerはenを自動付加する。
//   このためen_xxxではなく_xxxを引数に渡す(美しくないが)
//   xxxを渡すようにしないのはxxxが変数名と一致すると紛らわしいため
#define GetEMapOwner(x, y, name) (_GetEMapOwner(x, y, en##name))
#define _GetEMapOwner(x, y, en)                 \
                                 (emap_owner[current_emap][en][y][x] ?  \
                                  emap_owner[current_emap][en][y][x] : \
                                 emap_owner_static[en][y][x])



extern emap_e_t emap_static[MAP_HEIGHT][MAP_WIDTH];
extern emap_e_t emap[2][MAP_HEIGHT][MAP_WIDTH];
extern obj_t *emap_owner_static[en_ownered_end][MAP_HEIGHT][MAP_WIDTH];
extern obj_t *emap_owner[2][en_ownered_end][MAP_HEIGHT][MAP_WIDTH];
extern int current_emap;

void emap_init(void);
void setup_emap(void);
void clear_emap_owner(void);
void clear_emap(void);
void switch_current_emap(void);
void cp_or_static_emap(void);
void copy_emap(void);
void cp_static_emap(void);
void set_emap_static_box(int ltx, int lty, int width, int height, int emask);
void reset_emap_static_box(int ltx, int lty, int width, int height, int emask);
void set_emap_rel(obj_t *obj, int rx, int ry, int emask);
void set_emap_box(int ltx, int lty, int width, int height, int emask);
void set_emap_box_rel(obj_t *obj, int rltx, int rlty, int rrbx, int rrby,
                      int emask);
void reset_emap_box(int ltx, int lty, int width, int height, int emask);
void set_emap_owner_box(int ltx, int lty, int width, int height,
                        obj_t *obj, effect_no_t en);
void set_emap_owner_static_box(int ltx, int lty, int width, int height,
                               obj_t *obj, effect_no_t en);

void set_emap_static_blockbox(int ltx, int lty, int width, int height);
void reset_emap_static_blockbox(int ltx, int lty, int width, int height);
void set_emap_blockbox(int ltx, int lty, int width, int height);

int check_emap_rel(obj_t *obj, int rx, int ry, int emask);
int check_emap_box_any(int ltx, int lty, int width, int height, int emask);
int check_emap_obj_any(obj_t *obj, int emask);
int check_emap_pos_box_any(int ltx, int lty, int width, int height, int emask);
int check_emap_pos_obj_any(obj_t *obj, int emask);

int check_emap_box_any_corner(int ltx, int lty, int width, int height,
                              int emask);
int check_emap_obj_any_corner(obj_t *obj, int emask);
int check_emap_pos_box_any_corner(int ltx, int lty, int width, int height,
                                  int emask);
int check_emap_pos_obj_any_corner(obj_t *obj, int emask);
int check_emap_box_all_corner(int ltx, int lty, int width, int height,
                              int emask);
int check_emap_obj_all_corner(obj_t *obj, int emask);
int check_emap_box_any_rim(int ltx, int lty, int width, int height, int emask);
int check_emap_obj_any_rim(obj_t *obj, int emask);
int check_emap_box_any_rim_rel(obj_t *obj, int rltx, int rlty,
                               int rrbx, int rrby, int emask);
int check_emap_box_any_left(int ltx, int lty, int width, int height,
                            int emask);
int check_emap_box_any_right(int ltx, int lty, int width, int height,
                             int emask);
int check_emap_box_any_up(int ltx, int lty, int width, int height,
                          int emask);
int check_emap_box_any_down(int ltx, int lty, int width, int height,
                            int emask);
int check_emap_obj_any_left(obj_t *obj, int emask);
int check_emap_obj_any_right(obj_t *obj, int emask);
int check_emap_obj_any_up(obj_t *obj, int emask);
int check_emap_obj_any_down(obj_t *obj, int emask);













// !! 廃止？
int check_emap_dir(int emask, dir_t dir, obj_t *obj);
int check_emap_left(int emask, obj_t *obj);
int check_emap_right(int emask, obj_t *obj);
int check_emap_up(int emask, obj_t *obj);
int check_emap_down(int emask, obj_t *obj);
int check_emap_any_rim(int emask, obj_t *obj);
#endif







