#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <EzGraph.h>
#include "config.h"
#include "obj.h"
#include "obj_draw.h"
#include "obj_jumeme.h"
#include "emap.h"

void create_jumeme(obj_t *obj, int ox, int oy, dir_t dir){
    obj->width = JUMEME_WIDTH;
    obj->height = JUMEME_HEIGHT;
	SetAttr(obj, at_meat);
	SetAttr(obj, at_can_climb);
	SetAttr(obj, at_immune_flame);
    obj->dir = dir;
    obj->u.creature.move_x = (dir == di_left) ? -1 : 1;
    obj->motion = (dir == di_left) ? ii_jumeme_walk_l0 : ii_jumeme_walk_r0;
    //	obj->u.creature.is_walking = 1; // 開始直後に反転しないようにするため
    obj->u.creature.land_ofst = 1;
}

int set_action_jumeme(obj_t *obj){
	if (!IsAlive(obj)) return 0;
	if (IsStunned(obj)) return 0;
    if (IsLanded(obj)){	
        if (check_emap_box_any(Ltx(obj)+2, Lty(obj)-1, obj->width-4,
                                1, EM_ICE)){
            SetState(obj, os_jumeme_melting_upper);
			SetMoveX(obj, 0);
            return 0;
        }
//        if (obj->dir == di_left && check_emap_box_any(Ltx(obj)-1, Lty(obj), 1, obj->height, eb_ice)){
        if (check_emap_box_any(Ltx(obj)-1, Lty(obj), 1, obj->height, EM_ICE)){
            obj->dir = di_left; 
			SetState(obj, os_jumeme_melting_side);
			SetMoveX(obj, 0);
            return 0;
        }
//        if (obj->dir == di_right && check_emap_box_any(Rtx(obj)+1, Lty(obj), 1, obj->height, eb_ice)){
        if (check_emap_box_any(Rtx(obj)+1, Lty(obj), 1, obj->height, EM_ICE)){
            obj->dir = di_right;
			SetState(obj, os_jumeme_melting_side);
			SetMoveX(obj, 0);
            return 0;
        }
        if (check_emap_box_any(Ltx(obj)+2, Lby(obj)+1, obj->width-4,
                                1, EM_ICE)){
            SetState(obj, os_jumeme_melting_lower);
			SetMoveX(obj, 0);
            return 0;
        }
        SetState(obj, os_none);
		SetMoveXDir(obj);
        // 前ターンで歩けたらそのまま歩き続ける
        if (!IsWalkBlocked(obj)){
            printf("continue walking %d\n", MoveX(obj));
        }
        // 滑っている最中はなにもしない
        else if (obj->u.creature.is_sliding){
        }
        // 着地かつ歩けない場合はぶつかっているので反転する
        else {
            //            printf("turn %d\n", obj->dir);
            obj->dir = ReverseDir(obj->dir);
            SetMoveXDir(obj);
        }
    }
    else { // 落下中
        if (IsWalkBlocked(obj)){
            obj->dir = ReverseDir(obj->dir);
        }
        if (obj->dir == di_left){
            if (CheckEMap(Lbx(obj)-1, Lby(obj)+0, EM_BLOCK_DOWN) ||
                CheckEMap(Lbx(obj)-1, Lby(obj)+1, EM_BLOCK_DOWN))
                //                CheckEMap(Lbx(obj)-1, Lby(obj)+1, EM_EXCLUDE_UP))
            {
                SetMoveXDir(obj);
            }
            else {
                SetMoveX(obj, 0);
            }
        }
        else {
            if (CheckEMap(Rbx(obj)+1, Rby(obj)+0, EM_BLOCK_DOWN) ||
                CheckEMap(Rbx(obj)+1, Rby(obj)+1, EM_BLOCK_DOWN))
            {
                SetMoveXDir(obj);
            }
            else {
                SetMoveX(obj, 0);
            }
        }
    }
	return 0;
}
void set_motion_jumeme(obj_t *obj){
    switch (State(obj)){
	SkipDefaultCreatureMotions;
	case os_jumeme_melting_upper:
		if (obj->dir ==  di_left){
			obj->motion = (obj->motion == ii_jumeme_melt_ul0) ?
                ii_jumeme_melt_ul1 : ii_jumeme_melt_ul0;
		}
		else {
			obj->motion = (obj->motion == ii_jumeme_melt_ur0) ?
                ii_jumeme_melt_ur1 : ii_jumeme_melt_ur0;
		}
        break;
	case os_jumeme_melting_side:
		if (obj->dir ==  di_left){
			obj->motion = (obj->motion == ii_jumeme_melt_l0) ?
                ii_jumeme_melt_l1 : ii_jumeme_melt_l0;
		}
		else {
			obj->motion = (obj->motion == ii_jumeme_melt_r0) ?
                ii_jumeme_melt_r1 : ii_jumeme_melt_r0;
		}
        break;
	case os_jumeme_melting_lower:
		obj->motion = (obj->motion == ii_jumeme_melt_sit0) ? 
            ii_jumeme_melt_sit1 : ii_jumeme_melt_sit0;
        break;
    default:
        if (obj->u.creature.is_sliding){
            obj->motion = (obj->dir ==  di_left) ? 
                ii_jumeme_walk_l0 : ii_jumeme_walk_r0;
        }
		// 落下中は足を固定
        // !! ii_jumeme_slide_l/rはブロックに乗り移るとき足先が重なるので使わない
        else if (!obj->u.creature.is_landed){
            SetMotionDir(obj, ii_jumeme_walk_l0, ii_jumeme_walk_r0);
        }
        // 歩行時
        else {
            SetWalkMotion(obj);
            /*
            if (obj->dir == di_left){
                obj->motion = (obj->motion == ii_jumeme_walk_l0) ?
                    ii_jumeme_walk_l1 : ii_jumeme_walk_l0;
            }
            else if (obj->dir == di_right){
                obj->motion = (obj->motion == ii_jumeme_walk_r0) ?
                    ii_jumeme_walk_r1 : ii_jumeme_walk_r0;
            }
            */
        }
        break;
    }
}

void set_emap_jumeme(obj_t *obj){
    set_emap_creature(obj);
    if (!IsAlive(obj)) return;
    switch (State(obj)){
    case os_jumeme_melting_upper:
        set_emap_box(obj->x, obj->y-1, obj->width, obj->height+1, EM_FLAME);
        break;
    case os_jumeme_melting_side:
		if (obj->dir == di_left){
	        set_emap_box(obj->x-1, obj->y, obj->width+1, obj->height, EM_FLAME);
		}
		else {
        set_emap_box(obj->x, obj->y, obj->width+1, obj->height, EM_FLAME);
		}
			break;
    case os_jumeme_melting_lower:
        set_emap_box(obj->x, obj->y+1, obj->width, obj->height, EM_FLAME);
        break;
    default:
        set_emap_box(obj->x, obj->y, obj->width, obj->height, EM_FLAME);
        break;
    }
}
