/*
  ゲーム内で使うパラメータ。config.hと異なり変更するとゲームエンジンに影響する値。
*/
#ifndef PARAM_H
#define PARAM_H
#include "config.h"

#define STAGE_NO_LEN 3

// ゲーム時間
#define GAME_TIME_MAX 999
#define GAME_TIME_LEN 3
#define GAME_TIME_INIT 180
#define GAME_TIME_SCORE 2
#define GONDIN_SCORE 50
#define OBJECTIVE_SCORE 100

// 最小オブジェクトの大きさ(および初期配置座標の間隔)
#define GRID_SIZE 4
// シフトでグリッドサイズ分のスケーリングを行う時のシフト量
#define GRID_SCALE 2



// ジャンプで上昇可能な時間
//#define CHEE_JUMP_TIME (GRID_SIZE*4/FALL_SPEED)
#define CHEE_JUMP_TIME (GRID_SIZE * 3)
// ジャンプで滞空(上昇+水平移動)可能な時間
//#define CHEE_FLOAT_TIME (GRID_SIZE*8/FALL_SPEED)
//#define CHEE_FLOAT_TIME (GRID_SIZE * 8)
#define CHEE_FLOAT_TIME (GRID_SIZE * 6)

#define CHEE_JUMP_HEIGHT 12

#define GHOST_COUNT 16
// 食べるモーションに使うコマ数
#define CHEE_EAT_COUNT 13
#define CHEE_EXIT_COUNT 10
#define CHEE_CLEAR_COUNT 8
#define CHEE_SHOW_RESULT_COUNT 30
#define CHEE_PIERCE_COUNT 1
#define CHEE_BURN_COUNT 4
#define OJIGIRI_EAT_COUNT 10
#define OJIGIRI_SWALLOW_COUNT 2
#define OJIGIRI_DIGEST_COUNT 20
#define OJIGIRI_VOMIT_COUNT 12

#define OJIGIRI_SWING_COUNT  4

#define CREATURE_DEATH_COUNT 8

// 攻撃状態の総フレーム数
#define GAGGA_HIT_COUNT 8
//#define GAGGA_COOLDOWN_COUNT 8
// 攻撃状態に入ってからBLAST判定が生じるまでのフレーム数
#define GAGGA_BLAST_COUNT 1
// 炎を吐き終わってから次に炎を吐くまでのフレーム数
#define SUE_WAIT_COUNT 32
// 炎を吐き続けるフレーム数
#define SUE_BREATH_COUNT 24
// 炎の寿命
#define FLAME_COUNT 24
//#define FLAME_COUNT 16
//#define FLAME_LEN 8
#define IWALL_LIFE 6

#define PunchInitPosLeft(nya) (Ltx(nya)-3)
#define PunchInitPosRight(nya) (Rtx(nya)-PUNCH_WIDTH+3+1)

#define FREEZE_COUNT 2

#define CHU_WALK_COUNT 16
#define CHU_STAND_COUNT 16
#define CHU_EATING_COUNT 10
#define CHU_EAT_COUNT 24

// ぴぃひゃらぴっとの揺れる周期
#define PPIT_SWAY_PERIOD 32
// ぴぃひゃらぴっとのジャンプ動作フレーム数
#define PPIT_JUMP_COUNT 8
#define PPIT_JUMP_TIME (MAP_GRID_HEIGHT * GRID_SIZE)
//#define PPIT_JUMP_TIME (16)

#define NYUL_HIDING_COUNT  40 // 
#define NYUL_SHOWING_COUNT 13 // 
#define NYUL_PEEKING_INTERVAL  10 //
#define NYUL_PEEKING_NUM  1 //
#define NYUL_PEEKING_COUNT     (NYUL_PEEKING_INTERVAL*NYUL_PEEKING_NUM)  // 
#define NYUL_GROWING_COUNT 5
#define NYUL_GROWING_COUNT2 (52+32)
#define NYUL_GROWN_COUNT 20  // 一時的。あとで消す


    
#define NYUL_TURN_PERIOD 16

// 伸びるモーションに使うコマ数


#define BANG_SPEED 3

#define DROP_LONG_INTERVAL 24
#define DROP_INTERVAL      12
#define DROP_SHORT_INTERVAL 6 

#define MINGO_ATTACK_COUNT 24

// 各キャラの大きさ
#define CHEE_SIZE 8
#define GONDIN_SIZE 8
#define GONDIN_WIDTH 8
#define GONDIN_HEIGHT 8
#define HATOPO_SIZE 8
#define GAGGA_SIZE 8
#define YARAMASI_WIDTH 8
#define YARAMASI_HEIGHT 4
#define NYA_WIDTH 8
#define NYA_HEIGHT 16
// にゃあとパンチの垂直オフセット
#define PUNCH_YOFFSET 9
#define PUNCH_WIDTH 4
#define PUNCH_HEIGHT 3
#define OJIGIRI_WIDTH 8
#define OJIGIRI_HEIGHT 16
#define SUE_WIDTH 12
#define SUE_HEIGHT 8
#define FLAME_WIDTH 4
#define FLAME_HEIGHT 4
#define CHU_WIDTH 4
#define CHU_WALK_HEIGHT 3
#define CHU_STAND_HEIGHT 4
#define MINGO_SIZE 8
#define JUMEME_WIDTH 8
#define JUMEME_HEIGHT 8
#define SIGUKU_WIDTH 4
#define SIGUKU_HEIGHT 4
#define PPIT_WIDTH 8
#define PPIT_HEIGHT 8

#define NYUL_WIDTH 4
#define NYUL_HEIGHT_INIT 1



#define NYUL_HEIGHT_MAX 8
#define NYUL_HEIGHT_MIN 4

#define GOAL_WIDTH 8
#define GOAL_HEIGHT 8
#define SIGN_WIDTH 8
#define SIGN_HEIGHT 8


// リフトの横幅
#define LIFT_DEFAULT_WIDTH     (GRID_SIZE * 3)
// ※static_emapにブロッキング情報を書き込むオブジェクトは
//   後でset_grid_width_static()を呼んで短くしても書き込まれたemap情報は消えない。
//   すぐ右側が空間の場合、空間の位置にこのemapが残りblockingになってしまう
//   したがって、create時の幅はGRID_SIZEでなければならない
#define SWITCH_DEFAULT_WIDTH   (GRID_SIZE * 1)
#define TELEPORTER_DEFAULT_WIDTH   (GRID_SIZE * 1)

// グリッド単位でのステージの大きさ(表示される分)
#define MAP_GRID_WIDTH  30
#define MAP_GRID_HEIGHT 30

// 内部処理上、ステージ周囲に余分に確保する領域の大きさ(グリッド単位)
#define MAP_GRID_MARGIN 1
#define MAP_MARGIN (MAP_GRID_MARGIN*GRID_SIZE)

#define MAP_WIDTH  ((MAP_GRID_WIDTH+MAP_GRID_MARGIN*2) * GRID_SIZE)
#define MAP_HEIGHT ((MAP_GRID_HEIGHT+MAP_GRID_MARGIN*2) * GRID_SIZE)
#define MAP_LTX MAP_MARGIN
#define MAP_LTY MAP_MARGIN
#define MAP_RBX (MAP_WIDTH-MAP_MARGIN-1)
#define MAP_RBY (MAP_HEIGHT-MAP_MARGIN-1)

#define STAGE_TITLE_WIDTH 8
#define STAGE_TITLE_ROW_NUM 2
//#define STAGE_DESCRIPTION_WIDTH 8
//#define STAGE_DESCRIPTION_ROW_NUM 8
#define STAGE_SIGN_WIDTH 8
#define STAGE_SIGN_ROW_NUM 5

#define OBJECTIVE_NUM 3

// ※入力チェックフラグが32ビットなのでINは32が上限
#define HUB_IN_MAX 32
#define HUB_OUT_MAX 32


#endif

